% LSQFIT_CI  Calculation of confidence intervals for non-linear
% regression coefficients using residuals and Jacobian.
%
% Usage:
%   [db, b_lb, b_ub, sb] = lsqfit_ci(b, res, J)
%   [db, b_lb, b_ub, sb] = lsqfit_ci(b, res, J, alpha)
% Inputs:
%   b -- regression coefficients, res -- regression residuals
%   J -- Jacobian,
%   alpha -- optional argument, (1-p) for confidence interval
%   (default value is 0.05, i.e. 95% confidence interval)
% Outputs:
%   db -- confidence interval in the form b+-db
%   b_lb, b_ub -- lower and upper boundaries of the confidence interval
%   sb -- standard deviation for b coefficient
%
% See also LSQFIT_GN, LSQFIT_LM, LSQFIT_DL
function [db, b_lb, b_ub, sb] = lsqfit_ci(b, res, J, alpha)
if nargin < 4
    alpha = 0.05; % Default 95% confidence interval
end
b = b(:); % Transform to the column vector
res = res(:); % Transform to the column vector
f = (numel(res) - numel(b)); % Degrees of freedom
sigma2 = res'*res / f; % Regression dispersion unbiased estimation
c = sigma2 * inv(J'*J); % Covariance matrix
sb = sqrt(diag(c)); % Standard deviation
% b) confidence intervals
db = sb * tinv(1 - alpha/2, f); % Confidence intervals (95%)
b_lb = b - db; b_ub = b + db; % Lower/upper boundaries
end